import { Component, OnInit, ViewChild, ViewContainerRef, ComponentFactoryResolver } from '@angular/core';
import { Router, NavigationEnd } from '@angular/router';
import { ConfirmationService } from 'primeng/primeng';

// Global Common
import { BaseSession } from '@shared/index-service';
// ------------------

// Global Components
import { NotFound, AuditComponent, LoaderComponent } from '@shared/index';
// ------------------

// Fileman Components
import { FilemanDashboardComponent, SitesComponent, DirectoriesComponent, MasksComponent, ApplicationParamsComponent,
         ReferenceCodesComponent, OriginsComponent, LogsComponent, ConfigurationComponent, QueryComponent,
         ProfilesComponent, AuditsComponent, StepComponent, TablesComponent,
         TableColumnsComponent, TableProfilesComponent } from '@fileman/index';
// ------------------

// Standard Error Messages
import {MessageService} from 'primeng/components/common/messageservice';
// ------------------

@Component({
    templateUrl: './fileman.component.html',
    providers: [ConfirmationService, BaseSession]
})
export class FilemanComponent implements OnInit {

    msg: MessageService;

    // Include controls
    @ViewChild('slide', { read: ViewContainerRef }) slide: ViewContainerRef;
    @ViewChild('main', { read: ViewContainerRef }) main: ViewContainerRef;
    @ViewChild('stepbar', { read: ViewContainerRef }) navbar: ViewContainerRef;
    @ViewChild('audit', { read: ViewContainerRef }) audit: ViewContainerRef; 
    @ViewChild('loader', { read: ViewContainerRef }) loader: ViewContainerRef;

    // load handlers
    // -----------------------------------
    constructor(private resolver: ComponentFactoryResolver, private router: Router, private baseSession: BaseSession) {

    }

    ngOnInit() {

        this.audit.createComponent(this.resolver.resolveComponentFactory(AuditComponent)).changeDetectorRef.detectChanges();
        this.loader.createComponent(this.resolver.resolveComponentFactory(LoaderComponent)).changeDetectorRef.detectChanges();
   
        let pageLoad = this.router.url.split('/')[3];
    
        const compMap = {
            "dashboard": FilemanDashboardComponent,
            "sites": SitesComponent,
            "directories": DirectoriesComponent,
            "params": ApplicationParamsComponent,
            "origins": OriginsComponent,
            "refcodes": ReferenceCodesComponent,
            "masks": MasksComponent,
            "profiles": ProfilesComponent,
            "audits": AuditsComponent,
            "logs": LogsComponent,
            "tables": TablesComponent,
            "tableColumns": TableColumnsComponent,
            "tableProfiles": TableProfilesComponent,
            "configuration": ConfigurationComponent,
            "query": QueryComponent                 
          };

          try {
            pageLoad = pageLoad.split('?')[0];
            
            // if page contains the word dashboard - Load the dashboard
            if (pageLoad.toLowerCase().indexOf('dashboard') >= 0) {
              pageLoad = "dashboard-nav";
            }
       
            // If page is available in list
            if (pageLoad in compMap) {
                
                if (pageLoad.toLowerCase().indexOf('configuration') >= 0) {
                    this.navbar.createComponent(this.resolver.resolveComponentFactory(StepComponent)).changeDetectorRef.detectChanges();
                }
                if (pageLoad.toLowerCase().indexOf('query') >= 0) {
                    this.navbar.createComponent(this.resolver.resolveComponentFactory(StepComponent)).changeDetectorRef.detectChanges();
                }

                this.main.createComponent(this.resolver.resolveComponentFactory(compMap[pageLoad])).changeDetectorRef.detectChanges();
            } else {
              this.main.createComponent(this.resolver.resolveComponentFactory(NotFound)).changeDetectorRef.detectChanges();
            }
      
          } catch(e) {
            console.error("Controller Load: " + e.message);
          }

    }
    // -----------------------------------

}
