import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Load other
import { ApplicationParameter, ReferenceCode } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './application-parameters.component.html'
})

export class ApplicationParamsComponent extends PermissionComponent implements OnInit {

    // select data type dropdown
    // ----------------------------------
    dataTypes: ReferenceCode[] = [];
    dataTypesActive: ReferenceCode[] = [];

    formats: ReferenceCode[] = [];
    formatsActive: ReferenceCode[] = [];
    // ----------------------------------

    // validation  
    form: FormGroup;
    submitted: boolean;
    output = '';
    // ----------------------------------
    
    // grid details
    //  ----------------------------------
    appParams: ApplicationParameter[] = [];
    appParam: ApplicationParameter = new ApplicationParameter();
    //  ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private fb: FormBuilder) {
        super(baseService)
        
        // Check permission on function calls in save method
        this.baseService.checkPermission('ApplicationParameter', true);
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // form validation
        this.form = new FormGroup({
            'id': new FormControl({disabled: true}),
            'parameterName': new FormControl({}, [Validators.required, Validators.maxLength(50)]),
            'parameterValue': new FormControl({}, [Validators.required, Validators.maxLength(100)]),
            'parameterDataTypeCMId': new FormControl({}, [Validators.required, Validators.maxLength(10)]),
            'parameterFormatCMId': new FormControl({}, [Validators.required, Validators.maxLength(30)]),
            'parameterDescription': new FormControl({}, [Validators.required, Validators.maxLength(500)]),
            'isActive': new FormControl({}, [Validators.required])
        });

        // Call to local getGridList();
        this.getGridList();

        // Call to local getDataTypeSelectValues();
        this.getDataTypeSelectValues();
        this.getFormatSelectValues();
    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_done() {
        this.detail = false;
    }

    btn_back() {
        this.edit = false;
        this.form.reset();
        this.baseService.showAudit(false);
    }

    btn_new() {
        this.edit = true;
        this.form.reset();
        this.baseService.showAudit(false);
        this.edit = true;              
        this.form.reset();

        // Reset all items to null - empty inputs
        // ---------------------------------------                
        this.appParam = new ApplicationParameter();
        this.appParam.isActive = true;
        // ---------------------------------------
    }

    btn_save() {
      this.saveDetails();
    }
    // -----------------------------------

    // basic select events
    // -----------------------------------
    btn_select_row(index: number) {
        
        // Audit information
        this.baseService.showAudit(true);
        this.baseService.auditTable = this.appParams[index].domainName
        this.baseService.auditId = this.appParams[index].id;
        // ------------------------------------------------------------------

        // Determine object from grid row index
        this.getGridDetails(this.appParams[index].id);
    }

    btn_show_row(index: number) {
        this.appParam = this.appParams[index] as ApplicationParameter;
        this.output = JSON.stringify(this.appParam);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");
        this.output = this.output.replace(/":/g, '</strong><i>');
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
   }

    onSelectDataType(dataTypeId: string) {
        if (dataTypeId && dataTypeId != null && dataTypeId !== "-- select --") {
            this.appParam.parameterDataTypeCMId = +dataTypeId;
        } else {
            this.appParam.parameterDataTypeCMId = undefined;
        }        
    }

    onSelectFormat(formatId: string) {
        if (formatId && formatId != null && formatId !== "-- select --") {
            this.appParam.parameterFormatCMId = +formatId;
        } else {
            this.appParam.parameterFormatCMId = undefined;
        }        
    }
    // -----------------------------------

    // Data calls
    // -----------------------------------
    getDataTypeSelectValues() {
        this.baseService.getData('GET', 'referenceCodes/domain/DATA_TYPE', null /* searchParams */, false /* paging */,
                null /* updateParams */, true /* crossSectionOverride */, 'admin' /* crossSectionAPI */)
            .subscribe(
                data => {
                    this.dataTypes = data.json().content as ReferenceCode[];
                    
                    // create active list only
                    for (let i = 0; i < this.dataTypes.length; i++) {
                        if (this.dataTypes[i].isActive) {
                            this.dataTypesActive.push(this.dataTypes[i]);
                        }
                    }
                    // --------------------------------------------------------
                    
                }
            );
    }

    getFormatSelectValues() {
        this.baseService.getData('GET', 'referenceCodes/domain/DATE_MASK', null /* searchParams */, false /* paging */,
                null /* updateParams */, true /* crossSectionOverride */, 'admin' /* crossSectionAPI */)
            .subscribe(
                data => {
                    this.formats = data.json().content as ReferenceCode[];

                    // create active list only
                    for (let i = 0; i < this.formats.length; i++) {
                        if (this.formats[i].isActive) {
                            this.formatsActive.push(this.formats[i]);
                        }
                    }
                    // --------------------------------------------------------
                }
            );
    }

    getGridList() {
        this.baseService.getData('GET', 'applicationParameters?size=600&sort=parameterName')
            .subscribe(
                data => {
                    this.appParams = data.json().content as ApplicationParameter[];
                    this.nodata = false;
                    this.edit = false;
                }
            );
    }

    setupFormValues(): void {
        this.form.get( 'id').setValue(this.appParam.id);
        this.form.get( 'parameterName').setValue(this.appParam.parameterName);
        this.form.get( 'parameterValue').setValue(this.appParam.parameterValue);

        if (this.appParam.parameterDataTypeCMId) {
            this.form.get( 'parameterDataTypeCMId').setValue(this.appParam.parameterDataTypeCMId);
        } else {
            this.form.get( 'parameterDataTypeCMId').setValue('undefined');
        }

        if (this.appParam.parameterFormatCMId) {
            this.form.get( 'parameterFormatCMId').setValue(this.appParam.parameterFormatCMId);
        } else {
            this.form.get( 'parameterFormatCMId').setValue('undefined');
        }

        this.form.get( 'parameterDescription').setValue(this.appParam.parameterDescription);
        this.form.get( 'isActive').setValue(this.appParam.isActive);
        

    }

    getGridDetails(id?: number) {
        if (id && id != null) {
            this.baseService.getData('GET', 'applicationParameters/' + id)
                .subscribe(
                    data => {
                        this.appParam = data.json() as ApplicationParameter;

                        this.setupFormValues();

                        this.nodata = false;
                        this.edit = true;
                    }
                );
        }
    }

    saveDetails() {
        const method: string = this.baseService.checkMethod(this.appParam.id);
        const url: string = 'applicationParameters' + this.baseService.checkCreate(this.appParam.id);
        
        let parName = '';
        parName = this.form.get('parameterName').value;
        parName = parName.toUpperCase();

        this.form.get('parameterName').setValue(parName);
        
        const jsonValue: string = JSON.stringify(this.form.value);

        // Update / Create items
        // -----------------------------------
        this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
           .subscribe(
               (data) => {
                    this.baseService.handleErrorLog('success', 'Your Application Parameter was saved.', 'Saved');
                    this.appParam = data.json() as ApplicationParameter;
                    this.getGridList();
                    this.edit = false;
               },
               (error: any) => {
                    this.baseService.handleErrorLog('error', 'Saving of the Application Parameter failed!', error);
               }
           );
        // -----------------------------------
    }
    // -----------------------------------
}
