import { Injectable } from '@angular/core';

// Global Common
import { FilemanSession } from '@fileman/index-domain';
import { BaseSession } from '@shared/index-service';
import { BaseEnvironment } from '@shared/common/services/base.environment';
import { Session } from '@shared/index-domain';
// ------------------

@Injectable()
export class FilemanSessionService {

    filemanSessions: FilemanSession[] = [];
    filemanSessionsCopy: FilemanSession[] = [];
    filemanSession: FilemanSession;

    filemanSessionId = 1;
    filemanSessionDescription = 'fileman';

    maxSteps = 2;

    constructor(public session: BaseSession) {
    }

    public setSession(id: number, description: string, step: string, stepAdditional?: boolean) {

            try {

                const filemanSession: FilemanSession = new FilemanSession();
                filemanSession.id = id;
                filemanSession.description = description;
                filemanSession.stepDescription = step;

                // Show option for record types
                if (stepAdditional) {
                    filemanSession.stepAdditional = true;
                }
                // -----------------------------------------
                
                this.filemanSessions = this.getSessionFull();
                this.filemanSessionsCopy = [];

                // check if current item exist
                let exist = false;

                filemanSession.selected = true;

                try {
                    for (let i = 0; i <= this.filemanSessions.length; i++) {
                        this.filemanSessions[i].selected = false;
                        this.filemanSessions[i].stepPosition = i;

                        if (this.filemanSessions[i].stepDescription === filemanSession.stepDescription) {
                            exist = true;
                            filemanSession.stepPosition = i + 1;
                            this.filemanSessions[i] = filemanSession;
                        }

                        // Remove items greater than current step (Refresh)

                        //  ---------------------------------------------------------------
                    }

                    this.filemanSessions = this.filemanSessionsCopy;

                } catch (e) {

                }

                    // Only add item if it doesnt exist already
                    if (!exist) {
                        filemanSession.stepPosition = this.filemanSessions.length;
                        this.filemanSessions.push(filemanSession);
                        this.session.setSession(this.filemanSessionId, this.filemanSessionDescription, JSON.stringify(this.filemanSessions), false);
                    } else { // Update session item
                        this.session.setSession(this.filemanSessionId, this.filemanSessionDescription, JSON.stringify(this.filemanSessions), false);
                    }
                    
            } catch (e) {

            }
    

    }

    public removeSession(key: string) {

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].stepDescription === key) {
                    this.filemanSessions.splice(i, 1);
                }
            }

        } catch (e) {

        }

        this.session.setSession(this.filemanSessionId, this.filemanSessionDescription, JSON.stringify(this.filemanSessions), false);

    }

    public getSession(id: number): FilemanSession {

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].id === id) {
                    this.filemanSession = this.filemanSessions[i];
                }
            }

        } catch (e) {

        }

        return this.filemanSession;
    }

    public checkSessionIdWithKey(key: string): boolean {

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].stepDescription === key) {
                    return true;
                }
            }

        } catch (e) {

        }

        return false;
    }

    public checkSessionAdditionalStepWithKey(key: string): boolean {

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].stepAdditional) {
                    return true;
                }
            }

        } catch (e) {

        }

        return false;
    }
    public getSessionLabelWithKey(key: string): string {

        let label = '';

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].stepDescription === key) {
                    if (this.filemanSessions[i].id != null) {
                        label = ' - ' + this.filemanSessions[i].description + '[' + this.filemanSessions[i].id.toString() + ']';
                    }
                }
            }

        } catch (e) {

        }

        return label;
    }

    public getSessionIdWithKey(key: string): number {

        let id = 0;

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].stepDescription === key) {
                    id = this.filemanSessions[i].id;
                }
            }

        } catch (e) {

        }

        return id;
    }

    public getSessionIdCurrentActive(): number {

        let position = 0;

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].selected) {

                    if (this.filemanSessions[i].id === null) {
                        position = i
                    } else {
                        position = i + 1;
                    }
                }
            }

        } catch (e) {
            
        }

        return position;
    }

    public getSessionIdPreviousActive(): number {

        let previousPosition = 0;

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i < this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].selected) {
                         previousPosition = i - 1;
                }

                this.filemanSessions[i].selected = false;
            }

            if (previousPosition < 0) {
                previousPosition = 0;
                this.filemanSessions[previousPosition].id = null;
            }

            this.filemanSessions[previousPosition].selected = true;

        } catch (e) {
            
        }

        // clear all items greater than current
        for (let i = this.filemanSessions.length; i >= 0; i--) {
            if (i > previousPosition) {
                this.filemanSessions.splice(i, 1);
            }
        }
        // ------------------------------------------------

        // Update session
        this.session.setSession(this.filemanSessionId, this.filemanSessionDescription, JSON.stringify(this.filemanSessions), false);

        return previousPosition;
    }

    public getSessionIdNextActive(): number {

        let nextPosition = 4;

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i < this.filemanSessions.length; i++) {

                if (this.filemanSessions[i].selected) {
                    nextPosition = i + 1;
                }

                this.filemanSessions[i].selected = false;
            }

            if (nextPosition > 4) {
                nextPosition = 4;
            }

            this.filemanSessions[nextPosition].id = null;
            this.filemanSessions[nextPosition].selected = true;

        } catch (e) {
            
        }

        // Update session
        this.session.setSession(this.filemanSessionId, this.filemanSessionDescription, JSON.stringify(this.filemanSessions), false);

        return nextPosition;
    }

    public setSessionCurrentActive(key: string) {

        let currentPostition = 0;

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            for (let i = 0; i <= this.filemanSessions.length; i++) {

                try {
                    this.filemanSessions[i].selected = false;
                } catch (e) {

                }

                if (this.filemanSessions[i].stepDescription === key) {
                    this.filemanSessions[i].id = null
                    this.filemanSessions[i].selected = true;
                    currentPostition = i;
                }
            }

        } catch (e) {

        }

        for (let i = this.filemanSessions.length; i >= 0; i--) {
                // clear all items greater than current
                if (i > currentPostition) {
                    this.filemanSessions.splice(i, 1);
                }
            }


        this.session.setSession(this.filemanSessionId, this.filemanSessionDescription, JSON.stringify(this.filemanSessions), false);

    }

    public getSessionFull(): FilemanSession[] {

        try {

            const jsn: string = JSON.stringify(this.session.getSession(this.filemanSessionId).jsonString);
            this.filemanSessions = JSON.parse(jsn);

            //JSON.stringify(this.session.getSessionFull());


        } catch (e) {

        }

        return this.filemanSessions;
    }

}
