import {Injectable} from '@angular/core';
import { Router } from '@angular/router';

// Global Common
import { BaseService } from '@shared/index-service';
// ------------------

// Fileman Common
import { Step, FilemanSession } from '@fileman/index-domain';
import { FilemanSessionService } from '@fileman/common/service/session.service';
// ------------------

@Injectable()
export class StepNavigationService {

    session: FilemanSession[] = [];

    navigation: Step[];
    activeIndex = 1;
    application = '';
    page = '';

    constructor(private router: Router, private baseService: BaseService, private filemanService: FilemanSessionService) {

    }

    public getSteps(): Step[] {

        // Get current session values
        this.session = this.filemanService.getSessionFull();
        // -----------------------------------

        // Determine details from url
        this.application = this.router.url.split('/')[2].toLowerCase();
        this.page = this.router.url.split('/')[3].toLowerCase();
        // -------------------------------------------------------

        // Load different steps for different sections
        if (this.page === 'configuration') {
             this.configuration();
        } else if (this.page === 'query') {
             this.query();
        }
        // -------------------------------------------------------

        return this.navigation;
    }

    private configuration() {

        this.navigation = [];

        this.navigation.push({
            label: 'Origin' + this.filemanService.getSessionLabelWithKey('origin') , step: 0, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('origin');
                this.baseService.refreshPage();
            }, stepKey: 'origin'
        });
        this.navigation.push({
            label: 'File Masks' + this.filemanService.getSessionLabelWithKey('mask'), step: 1, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('mask');
                this.baseService.refreshPage();
            }, stepKey: 'mask'
        });
        this.navigation.push({
            label: 'File Profiles' + this.filemanService.getSessionLabelWithKey('profile'), step: 2, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('profile');
                this.baseService.refreshPage();
            }, stepKey: 'profile'
        });
        this.navigation.push({
            label: 'Types' + this.filemanService.getSessionLabelWithKey('types'), step: 3, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('types');
                this.baseService.refreshPage();
            }, stepKey: 'types'
        });

        // If session id with key 'record' is populated it means there is multiple record types
        // if (this.filemanService.checkSessionAdditionalStepWithKey('record')) {
        //     this.navigation.push({
        //         label: 'Record Types' + this.filemanService.getSessionLabelWithKey('record'), step: 4, command: (event: any) => {
        //             this.filemanService.setSessionCurrentActive('record');
        //             this.baseService.refreshPage();
        //         }, stepKey: 'record'
        //     });
        // }
         // ---------------------------------------------------

    }

    private query() {

        this.navigation = [];

        this.navigation.push({
            label: 'Origin' + this.filemanService.getSessionLabelWithKey('origin'), step: 0, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('origin');
                this.baseService.refreshPage();
            }, stepKey: 'origin'
        });
        this.navigation.push({
            label: 'File Masks' + this.filemanService.getSessionLabelWithKey('mask'), step: 1, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('mask');
                this.baseService.refreshPage();
            }, stepKey: 'mask'
        });
        this.navigation.push({
            label: 'File Profiles' + this.filemanService.getSessionLabelWithKey('profile'), step: 2, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('profile');
                this.baseService.refreshPage();
            }, stepKey: 'profile'
        });
        this.navigation.push({
            label: 'Files' + this.filemanService.getSessionLabelWithKey('files'), step: 3, command: (event: any) => {
                this.filemanService.setSessionCurrentActive('files');
                this.baseService.refreshPage();
            }, stepKey: 'files'
        });
    }
}
