import { Component, OnInit } from '@angular/core';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { AuditOutputFile, AuditOutputFileDescriptor, FileAuditDescriptor } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './audit-output-files.component.html'
})

export class LogsComponent extends PermissionComponent implements OnInit {

    // form
    search = false;
    stepKey = 'log';

    fileText = '';
    fileAuditId: number = undefined;
    fileAudits: FileAuditDescriptor[] = [];
    // ----------------------

    // validation  
    output = '';
    // ----------------------------------

    // grid details
    //  ----------------------------------
       auditOutputFiles: AuditOutputFileDescriptor[] = [];
       auditOutputFile: AuditOutputFile = new AuditOutputFile();
    //  ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private sessionService: FilemanSessionService) {
        super(baseService);
        
        // Check permission on function calls in save method
        this.baseService.checkPermission('AuditOutputFile');
        // -------------------------------------------------------------------
        
        // Override Create for query only
        if (this.baseService.getPage().toLowerCase() === 'query') {
            this.baseService.create = false;
        }
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // Retrieve values from session
        this.search = this.sessionService.checkSessionIdWithKey(this.stepKey);
        this.fileAuditId = this.sessionService.getSessionIdWithKey('audit');
        // -----------------------------------------------

        // Load data
        if (!this.fileAuditId) {
            this.getFileAuditsList();
        }

        this.getGridList(this.fileAuditId);
        // -----------------------------------------------

    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_back() {
        this.edit = false;
        this.baseService.showAudit(false);
    }

    btn_new() {
        this.edit = true;
        this.baseService.showAudit(false);

        // Reset all items to null - empty inputs
        // ---------------------------------------

        // ---------------------------------------

    }

    btn_save() {
    //  this.saveDetails();
    }

    btn_viewContent() {

        this.baseService.getData('GET', 'files/auditOutputFiles/' + this.auditOutputFile.id + '/content')
            .subscribe(
                (data) => {
                    alert(data.text());
                }
            );

    }

    btn_show_row(id: number) {
        this.auditOutputFile = this.auditOutputFiles[id] as AuditOutputFile;

        this.output = JSON.stringify(this.auditOutputFile);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
        this.output = this.output.replace(/":/g, '</strong><i>');   
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
    }

    btn_done() {
        this.detail = false;
    }

    onRowSelect() {
       this.getGridDetails(this.auditOutputFile.id);

       this.baseService.getData('GET', 'files/auditOutputFiles/' + this.auditOutputFile.id + '/content')
            .subscribe(
                (data) => {
                    this.fileText = data.text();
                }
            );
    }

    onSelectFileAudit(id: number) {
        this.getGridList(id);
    }
    // -----------------------------------

    // Data calls
    // -----------------------------------
    getFileAuditsList() {
        this.baseService.getData('GET', 'files/fileAudits?size=600&sort=id,desc')
            .subscribe(
                (data) => {
                    this.fileAudits = data.json().content as FileAuditDescriptor[];
                    this.nodata = false;
                }
            );
    }

    getGridList(id?: number) {

        let endpoint = 'files/auditOutputFiles?size=600&sort=id,desc';
        if (id && id != null) {
            endpoint = 'files/auditOutputFiles/fileAudit/' + id + '?size=600&sort=id,desc'; // include id
        }

        this.baseService.getData('GET', endpoint)
            .subscribe(
                data => {
                    this.auditOutputFiles = data.json().content as AuditOutputFileDescriptor[];
                    this.nodata = false;
                    this.edit = false;
                }
            );
    }

    getGridDetails(id?: number) {

        if (id != null) {
            this.baseService.getData('GET', 'files/auditOutputFiles/' + id)
                .subscribe(
                    data => {
                        this.auditOutputFile = data.json() as AuditOutputFile;
                        this.nodata = false;
                        this.edit = true;
                    }
                );
        }

    }

    saveDetails() {

        // Update / Create items
//        this.baseService.getData(this.baseService.checkMethod(this.entity.id),
//  'URL to Post/Put to' + this.baseService.checkCreate(this.entity.id), null, false, 'JSON.string')
//            .subscribe(
//                data => {
//                    //this.edit = false;
//                }
//            );
//
        this.baseService.handleErrorLog('success', 'Your details were updated', 'Saved')
        // -----------------------------------

    }
    // -----------------------------------
}

