import { Component, OnInit } from '@angular/core';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { FileAuditDescriptor, FileAudit } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './file-audits.component.html'
})

export class AuditsComponent extends PermissionComponent implements OnInit {

    // form
    search = false;
    stepKey = 'audit';
    fileProfileId: number = undefined;

    // validation  
    output = '';
    // ----------------------------------

    // grid details
    //  ----------------------------------
       fileAudits: FileAuditDescriptor[] = [];
       fileAudit: FileAudit = new FileAudit();
    //  ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private sessionService: FilemanSessionService) {
        super(baseService);
        
        // Check permission on function calls in save method
        this.baseService.checkPermission('FileAudit');
        // -------------------------------------------------------------------
        
        // Override Create for query only
        if (this.baseService.getPage().toLowerCase() === 'query') {
            this.baseService.create = false;
        }
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // Retrieve values from session
        this.search = this.sessionService.checkSessionIdWithKey(this.stepKey);
        this.fileProfileId = this.sessionService.getSessionIdWithKey('profile');
        // -----------------------------------------------

        // Load data
        this.getGridList(this.fileProfileId);
        // -----------------------------------------------
    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_back() {
        this.edit = false;
        this.baseService.showAudit(false);
    }

    btn_next_step() {
        this.sessionService.setSession(this.fileAudit.id, this.fileAudit.fileName, this.stepKey);
        this.baseService.refreshPage();
    }

    btn_show_row(id: number) {
        this.fileAudit = this.fileAudits[id] as FileAudit;

        this.output = JSON.stringify(this.fileAudit);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
        this.output = this.output.replace(/":/g, '</strong><i>');   
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
    }

    btn_done() {
        this.detail = false;
    }

    onRowSelect() {
        this.sessionService.setSession(this.fileAudit.id, this.fileAudit.fileName, this.stepKey);
        this.baseService.refreshPage();
    }

    // -----------------------------------
    // Data calls
    // -----------------------------------
    getGridList(id?: number) {

        let endpoint = 'files/fileAudits?size=600&sort=id,desc';
        if (id && id != null) {
            endpoint = 'files/fileAudits/fileProfile/' + id + '?size=600&sort=id,desc';
        }
        this.baseService.getData('GET', endpoint)
            .subscribe(
                data => {
                    this.fileAudits = data.json().content as FileAuditDescriptor[];
                    this.nodata = false;
                    this.edit = false;
                }
            );
    }
    // -----------------------------------
}

