import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { FileAudit, AuditOutputFile, AuditOutputFileDescriptor, FileProcessException, FileProcessExceptionDescriptor } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './file-dashboard.component.html'
})

export class FileDashboardComponent extends PermissionComponent implements OnInit {

    // form
    search = false;
    stepKey = 'audit';
    fileProfileId: number = undefined;
    activeIndex = 0;
    showFileDetails = false;
    showFile = false;

    // validation  
    output = '';
    // ----------------------------------

    // dates
    startDate: Date;
    endDate: Date;
    dateDiffAllowed = 5; //Days
    // ----------------------

    // grid details
    //  ----------------------------------
       fileAudits: FileAudit[] = [];
       fileAudit: FileAudit = new FileAudit();

       auditOutputFiles: AuditOutputFileDescriptor[] = [];
       auditOutputFile: AuditOutputFile = new AuditOutputFile();

       fileProcessExceptions: FileProcessExceptionDescriptor[] = [];
       fileProcessException: FileProcessException = new FileProcessException();

       fileText = '';
    //  ----------------------------------

    // validation  
    form: FormGroup;
    submitted: boolean;
    // ----------------------------------
        
    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private sessionService: FilemanSessionService) {
        super(baseService);
        
        // Check permission on function calls in save method
        this.baseService.checkPermission('FileAudit');
        // -------------------------------------------------------------------
        
        // Override Create for query only
        if (this.baseService.getPage().toLowerCase() === 'query') {
            this.baseService.create = false;
        }
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // Retrieve values from session
        this.search = this.sessionService.checkSessionIdWithKey(this.stepKey);
        this.fileProfileId = this.sessionService.getSessionIdWithKey('profile');
        // -----------------------------------------------

        this.startDate = new Date(Date.now());
        this.startDate.setDate( this.startDate.getDate() - this.dateDiffAllowed );
    
        this.endDate = new Date(Date.now());   

        // form validation
        this.form = new FormGroup({
            'startDate': new FormControl({}, Validators.required),
            'endDate': new FormControl({}, Validators.required)
        });

        this.form.get( 'startDate').setValue(this.startDate);
        this.form.get( 'endDate').setValue(this.endDate);

        // Load data
        //this.getGridList(this.fileProfileId);
        // -----------------------------------------------
    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_back() {
        this.edit = false;
        this.baseService.showAudit(false);
    }

    btn_next_step() {
        this.sessionService.setSession(this.fileAudit.id, this.fileAudit.fileName, this.stepKey);
        this.baseService.refreshPage();
    }

    btn_show_row(id: number) {
        this.fileAudit = this.fileAudits[id] as FileAudit;

        this.output = JSON.stringify(this.fileAudit);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
        this.output = this.output.replace(/":/g, '</strong><i>');   
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
    }

    btn_bcak() {
        this.edit = false;
    }

    btn_done() {
        this.detail = false;
    }

    btn_view(selectedAudit: FileAudit) {
       
                
        //setTimeout( () => {  // Pause for a couple of seconds to allow user to see change
         //}, 1000 );

        if (selectedAudit.parentFileId !== null) {
            this.showFileDetails = true;
            this.edit = true;
            this.fileAudit = null;
            
            this.activeIndex = 0;
            this.fileAudit = new FileAudit();
            this.fileAudit = selectedAudit;
                
            if (this.fileAudits.length > 1) {
                this.fileAudit.selected = true;
                for (let i = 0; i < this.fileAudits.length; i++) {
                    this.fileAudits[i].selected = false;
                    if (this.fileAudits[i].id === this.fileAudit.id) {
                        this.fileAudits[i].selected = true;
                    }
                }
            }

            this. getOutputFileList(this.fileAudit.id);
            this.getExceptionList(this.fileAudit.id);       
        } else {
            this.baseService.handleErrorLog('error', 'Parent files not available.', 'Failed');
        }

    }

    btn_open(selectedAudit: FileAudit) {
        
        this.fileAudit = selectedAudit;
        
        try {
        this.baseService.getData('GET', 'files/fileAudits/' + this.fileAudit.id + '/content')
             .subscribe(
                 (data) => {
                     this.fileText = data.text();
                     this.showFile = true;
                 }
             );
            } catch(e) {
               
            }
    }

    btn_select_audit(index: number) {
        //this.getOutputFileDetails(this.auditOutputFiles[index].id);
 
        this.baseService.getData('GET', 'files/auditOutputFiles/' + this.auditOutputFiles[index].id + '/content')
             .subscribe(
                 (data) => {
                     this.fileText = data.text();
                     this.showFile = true;
                 }
             );
     }

     btn_select_exception(index: number) { 
        this.fileText =this.fileProcessExceptions[index].exceptionText;
        this.showFile = true;
     }

     btn_close_file() {
        this.showFile = false;
     }
     
     btn_filter() {
        
        this.endDate = new Date(this.form.get( 'endDate').value);
        this.startDate = new Date(this.form.get( 'startDate').value);

        const dateDiff = new Date(this.endDate);
        dateDiff.setDate(dateDiff.getDate() - this.dateDiffAllowed);
        
        this.fileAudits = [];
        
        if (this.endDate < this.startDate) {
            this.baseService.handleErrorLog('error', 'End date can not be smaller than start date.', 'Failed');
        } else if (this.startDate < dateDiff) {            
            this.baseService.handleErrorLog('error', 'You are only allowed to do a ' + this.dateDiffAllowed + ' day search.', 'Failed');
        } else {
            this.getGridList(this.fileProfileId);
        }        
        
     }

    onRowSelect() {
        this.sessionService.setSession(this.fileAudit.id, this.fileAudit.fileName, this.stepKey);
        this.baseService.refreshPage();
    }

    // -----------------------------------
    // Data calls
    // -----------------------------------
    getGridList(id?: number) {

        let endpoint = 'files/fileAudits?size=600&sort=id,asc';
        if (id && id != null) {
            endpoint = 'files/fileAudits/fileProfile/' + id + '?size=2000&sort=id,parentFileId,asc';
        }
           
        this.baseService.getData('GET', endpoint)
            .subscribe(
                data => {
                    this.fileAudits = data.json().content as FileAudit[];                                  

                    // Only display values within date ranges
                    for (let i = 0; i < this.fileAudits.length; i++) {
                        
                        try {
                            
                            const dateStr = this.fileAudits[i].loadDate.toDateString().substring(1, 10); // IE 11 - Fix
                            const effectiveDate = new Date(dateStr)                           
                            
                            if (this.startDate <= effectiveDate && this.endDate >= effectiveDate) {
                                // do nothing
                            } else {
                                this.fileAudits.splice(i);
                            }
                        } catch(e) {}
                    }
                    // --------------------------------------------------

                    if (this.fileAudits.length < 1) {
                        this.baseService.handleErrorLog('error', 'No records for the selected dates.', 'Failed');
                    }
                    this.nodata = false;
                    this.edit = false;
                }
            );
    }

    getOutputFileList(id?: number) {

        let endpoint = 'files/auditOutputFiles?sort=id,desc';
        if (id && id != null) {
            endpoint = 'files/auditOutputFiles/fileAudit/' + id + '?sort=id,desc';
        }

        this.baseService.getData('GET', endpoint)
            .subscribe(
                data => {
                    this.auditOutputFiles = data.json().content as AuditOutputFileDescriptor[];
                }
            );
    }

    getOutputFileDetails(id?: number) {

        if (id != null) {
            this.baseService.getData('GET', 'files/auditOutputFiles/' + id)
                .subscribe(
                    data => {
                        this.auditOutputFile = data.json() as AuditOutputFile;
                    }
                );
        }

    }

    getExceptionList(id?: number) {

        let endpoint = 'files/fileProcessExceptions?sort=id,desc';
        if (id && id != null) {
            endpoint = 'files/fileProcessExceptions/fileAudit/' + id + '?sort=id,desc';
        }

        this.baseService.getData('GET', endpoint)
            .subscribe(
                data => {
                    //this.auditOutputFiles = data.json().content as AuditOutputFileDescriptor[];
                    this.fileProcessExceptions = data.json().content as FileProcessExceptionDescriptor[];

                    if (this.fileProcessExceptions.length < 1) {
                        this.fileProcessExceptions.push({exceptionText: "This is a hardcoded string just to indicate that we retrieve sample data from the database. /n/nCurrently no records available. /n/nThis record will not show if there is records available.", exceptionType: "string", id: 0, modifiedDate: null, modifiedUser: "", createdDate: null, createdUser: "", domainName: ""})
                    }
                    //console.error(JSON.stringify(this.fileProcessExceptions))
                }
            );
        
        }
    
    // -----------------------------------
}