import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { FileOrigin, FileOriginDescriptor, ReferenceCode } from '@fileman/index-domain';
//  -----------------

@Component({
       templateUrl: './file-origins.component.html'
})

export class OriginsComponent extends PermissionComponent implements OnInit {

       // form
       // ----------------------
       modules: ReferenceCode[] = [];
       modulesActive: ReferenceCode[] = [];
       selectedModuleId: string;

       search = false;
       stepKey = 'origin';
       //  ----------------------------------

       // validation  
       form: FormGroup;
       submitted: boolean;
       output = '';
       // ----------------------------------

       // grid details
       //  ----------------------------------
       fileOrigins: FileOriginDescriptor[] = [];
       fileOrigin: FileOrigin = new FileOrigin();
       //  ----------------------------------

       // load handlers
       // -----------------------------------
       constructor(baseService: BaseService, private sessionService: FilemanSessionService, private fb: FormBuilder) {
            super(baseService);
            
            // Check permission on function calls in save method
            this.baseService.checkPermission('fileOrigin');
            // -------------------------------------------------------------------

            // Override Create for query only
            if (this.baseService.getPage().toLowerCase() === 'query') {
                this.baseService.create = false;
            }
            // -------------------------------------------------------------------
       }

       ngOnInit() {
              // call to Permission Component --> componentInit();
              super.componentInit();

              // Retrieve values from session
              this.search = this.sessionService.checkSessionIdWithKey(this.stepKey);
              // -----------------------------------------------

              // form validation
              this.form = new FormGroup({
                     'id': new FormControl({disabled: true}),
                     'origin': new FormControl({}, [Validators.required, Validators.maxLength(50)]),
                     'purpose': new FormControl({}, [Validators.required, Validators.maxLength(2000)]),
                     'moduleId': new FormControl({}, Validators.required),
                     'autoLoad': new FormControl({}, Validators.required),
                     'status': new FormControl({}, Validators.required)
              });

              // Load data
              this.getModulesList();
              this.getGridList();
              // -----------------------------------

       }
       // -----------------------------------

       // basic button events
       // -----------------------------------
       btn_back() {
            this.edit = false;
            this.form.reset();
            this.baseService.showAudit(false);
        }

        btn_new() {
            this.edit = true;
            this.form.reset();
            this.baseService.showAudit(false);

              // Auto-populate / default necessary fields  
              this.fileOrigin = new FileOrigin();
                if (this.selectedModuleId) {
                    this.fileOrigin.module = new ReferenceCode();
                    this.fileOrigin.module.id = +this.selectedModuleId;
                    this.form.get('moduleId').setValue(this.selectedModuleId);
                }

                this.fileOrigin.autoLoad = false;
                this.form.get('autoLoad').setValue(false);

                this.fileOrigin.isActive = true;
                this.form.get('status').setValue(true);
              // ---------------------------------------

       }

       btn_save() {
              this.saveDetails();

              this.sessionService.setSession(this.fileOrigin.id, this.fileOrigin.origin, this.stepKey);
              this.baseService.refreshPage();

       }

        btn_next_step() {

              this.sessionService.setSession(this.fileOrigin.id, this.fileOrigin.origin, this.stepKey);
              this.baseService.refreshPage();

       }

        btn_select_row(id: number) {

            // Determin object from grid row index
            this.fileOrigin = this.fileOrigins[id] as FileOrigin;
            
            if (this.baseService.getPage() === 'query') {
                this.sessionService.setSession(this.fileOrigin.id, this.fileOrigin.origin, this.stepKey);
                this.baseService.refreshPage();
            } else {
                    this.getGridDetails(this.fileOrigin.id);
            }

            // Audit information
            this.baseService.showAudit(true);
            this.baseService.auditTable = this.fileOrigin.domainName
            this.baseService.auditId = this.fileOrigin.id;
            // ------------------------------------------------------------------

       }

       btn_show_row(id: number) {
            this.fileOrigin = this.fileOrigins[id] as FileOrigin;

            this.output = JSON.stringify(this.fileOrigin);
            this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
            this.output = this.output.replace(/":"/g, '</strong><i>');
            this.output = this.output.replace(/",/g, '</i><br>');
            this.output = this.output.replace(/,/g, '</i><br>');
            this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
            this.output = this.output.replace(/":/g, '</strong><i>');   
            this.output = this.output.replace(/"/g, "<strong>");
            //console.error(this.output);
            this.output = this.output.replace(/{/g, '<br>');
            this.output = this.output.replace(/}/g, '</i>') + '</i><br>';

            // console.error(output);

            this.detail = true;
       }

       btn_done() {
            this.detail = false;
       }

       onSelectModule(moduleId: string) {
        if (moduleId) {
                this.selectedModuleId = moduleId;
                this.getGridList(+moduleId);
            } else {
                this.selectedModuleId = "";
                this.getGridList(null);
            }
        }
        
       // -----------------------------------
       // Data calls
       // -----------------------------------
       getModulesList() {
              this.baseService.getData('GET', 'referenceCodes/domain/MODULE')
                     .subscribe(
                            (data) => {
                                   this.modules = data.json().content as ReferenceCode[];
                                  
                                    // create active list only
                                    for (let i = 0; i < this.modules.length; i++) {
                                        if (this.modules[i].isActive) {
                                            this.modulesActive.push(this.modules[i]);
                                        }
                                    }
                                    // --------------------------------------------------------
                                   
                                   this.nodata = false;
                            }
                     );
       }

       getGridList(moduleId?: number) {

              let endpoint = 'files/fileOrigins?size=600&sort=origin';
              if (moduleId && moduleId != null) {
                     endpoint = 'files/fileOrigins/module/' + moduleId + '?size=600&sort=origin';
              }
              this.baseService.getData('GET', endpoint)
                     .subscribe(
                            data => {
                                   this.fileOrigins = data.json().content as FileOriginDescriptor[];                                   
                                   this.nodata = false;
                                   this.edit = false;
                            }
                     );
       }

       determineModelModuleId(): number {
              return (this.fileOrigin.module) ? this.fileOrigin.module.id : undefined;
       }

       getGridDetails(id: number) {
              if (id && id != null) {
                     this.baseService.getData('GET', 'files/fileOrigins/' + id)
                            .subscribe(
                                   data => {
                                          this.fileOrigin = data.json() as FileOrigin;

                                          this.form.get('id').setValue(this.fileOrigin.id);
                                          this.form.get('origin').setValue(this.fileOrigin.origin);
                                          this.form.get('purpose').setValue(this.fileOrigin.purpose);
                                          this.form.get('moduleId').setValue(this.fileOrigin.module.id);
                                          this.form.get('autoLoad').setValue(this.fileOrigin.autoLoad);
                                          this.form.get('status').setValue(this.fileOrigin.isActive);

                                          this.nodata = false;
                                          this.edit = true;
                                   }
                            );
              }
       }

       saveDetails() {
                
                // Populate fileOrigin object according to screen values / ensure values correspond.
                this.fileOrigin.id = this.form.get('id').value;
                this.fileOrigin.origin = this.form.get('origin').value;
                for (let i=0; i<this.modules.length; i++) {
                    if (this.modules[i].id === +this.form.get('moduleId').value) {
                        this.fileOrigin.module = this.modules[i];
                        break;
                    }
                }
                this.fileOrigin.purpose = this.form.get('purpose').value;
                this.fileOrigin.autoLoad = this.form.get('autoLoad').value;
                this.fileOrigin.isActive = this.form.get('status').value;

                const method: string = this.baseService.checkMethod(this.fileOrigin.id);
                const url: string = 'files/fileOrigins' + this.baseService.checkCreate(this.fileOrigin.id);
                const jsonValue: string = JSON.stringify(this.fileOrigin);
                
                const idMemory = (this.fileOrigin.module.id) ? this.fileOrigin.module.id : undefined;

                // Update / Create items
                this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
                        .subscribe(
                                (data) => {
                                    this.baseService.handleErrorLog('success', 'Your File Origin was updated.', 'Saved');
                                    this.fileOrigin = data.json() as FileOrigin;
                                    this.getGridList(idMemory);
                                    this.edit = false;
                                },
                                (error: any) => {
                                    this.baseService.handleErrorLog('error', 'Saving of the File Origin failed!', error);
                                }
                        );
                // -----------------------------------
       }

       // -----------------------------------
}

