import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Load other
import { FileProcessException, FileProcessExceptionDescriptor } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './file-process-exceptions.component.html'
})

export class ExceptionsComponent extends PermissionComponent implements OnInit {

    // form
    output = '';
    // ----------------------

    // grid details
    //  ----------------------------------
       fileProcessExceptions: FileProcessExceptionDescriptor[] = [];
       fileProcessException: FileProcessException = new FileProcessException();
    //  ----------------------------------

       // validation  
       form: FormGroup;
       submitted: boolean;
       // ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private fb: FormBuilder) {
        super(baseService);
        
        // Check permission on function calls in save method
        this.baseService.checkPermission('FileProcessException');
        // -------------------------------------------------------------------
        
        // Override Create for query only
        if (this.baseService.getPage().toLowerCase() === 'query') {
            this.baseService.create = false;
        }
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

              // form validation
              this.form = new FormGroup({
                     'id': new FormControl({disabled: true}),
                     'category': new FormControl(Validators.required), //Validators.compose([IPAddressValidator.validIP, ])
                     'extension': new FormControl(Validators.required),
                     'mask': new FormControl(Validators.required),
                     'delemiter': new FormControl(Validators.required),
                     'action': new FormControl(Validators.required),
                     'days': new FormControl(Validators.required),
                     'compressed': new FormControl(Validators.required),
                     'heading': new FormControl(Validators.required),
                     'lenght': new FormControl(Validators.required),
                     'types': new FormControl(Validators.required)
              });

        this.getGridList();

    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_back() {
        this.edit = false;
        this.form.reset();
        this.baseService.showAudit(false);
    }

    btn_new() {
        this.edit = true;
        this.form.reset();
        this.baseService.showAudit(false);

        // Reset all items to null - empty inputs
        // ---------------------------------------
        
        // ---------------------------------------

    }

    btn_save() {
      this.saveDetails();
    }
   
    btn_select_row(id: number) {

        // Determin object from grid row index
        this.fileProcessException = this.fileProcessExceptions[id] as FileProcessException;
        
        this.getGridDetails(this.fileProcessException.id);

   }

    btn_show_row(id: number) {
        this.fileProcessException = this.fileProcessExceptions[id] as FileProcessException;

        // Audit information
        this.baseService.showAudit(true);
        this.baseService.auditTable = this.fileProcessException.domainName
        this.baseService.auditId = this.fileProcessException.id;
        // ------------------------------------------------------------------

        this.output = JSON.stringify(this.fileProcessException);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
        this.output = this.output.replace(/":/g, '</strong><i>');   
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';

        this.detail = true;
    }

    btn_done() {
        this.detail = false;
    }
    // -----------------------------------

    // Data calls
    // -----------------------------------
    getGridList(id?: number) {

//        if (id != null) {
            this.baseService.getData('GET', '') // Still to be defined based on how we plan to access this data.
                .subscribe(
                    data => {
                        this.fileProcessExceptions = data.json().content as FileProcessExceptionDescriptor[];
                        // console.error(JSON.stringify(this.fileProfiles))
                        this.nodata = false;
                        this.edit = false;
                    }
                );
//        }

    }

    getGridDetails(id?: number) {

        if (id != null) {
            this.baseService.getData('GET', 'files/exceptions/' + id)
                .subscribe(
                    data => {
                        this.fileProcessException = data.json() as FileProcessException;

                        this.form.get( 'id').setValue(this.fileProcessException.id);
                        this.form.get( 'category').setValue(this.fileProcessException.id);
                        this.form.get( 'extension').setValue(this.fileProcessException.id);
                        this.form.get( 'mask').setValue(this.fileProcessException.id);
                        this.form.get( 'delemiter').setValue(this.fileProcessException.id);
                        this.form.get( 'action').setValue(this.fileProcessException.id);
                        this.form.get( 'days').setValue(this.fileProcessException.id);
                        this.form.get( 'compressed').setValue(this.fileProcessException.id);
                        this.form.get( 'heading').setValue(this.fileProcessException.id);
                        this.form.get( 'lenght').setValue(this.fileProcessException.id);
                        this.form.get( 'types').setValue(this.fileProcessException.id);
                        
                        this.nodata = false;
                        this.edit = true;
                    }
                );
        }

    }

    saveDetails() {

        // Update / Create items
//        this.baseService.getData(this.baseService.checkMethod(this.entity.id),
//  'URL to Post/Put to' + this.baseService.checkCreate(this.entity.id), null, false, 'JSON.string')
//            .subscribe(
//                data => {
//                    //this.edit = false;
//                }
//            );
//
        this.baseService.handleErrorLog('success', 'Your details were updated', 'Saved')
        // -----------------------------------

    }
    // -----------------------------------
}

