import { Component, OnInit } from '@angular/core';
import { SelectItem } from 'primeng/api';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { ReferenceCode } from '@fileman/index-domain';
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { FileProfile, FileProfileDescriptor, IncomingFileMask, IncomingFileMaskDescriptor, FileOriginDescriptor } from '@fileman/index-domain';
import { Application } from '@admin/index-domain';
//  -----------------

@Component({
       templateUrl: './file-profiles.component.html'
})

export class ProfilesComponent extends PermissionComponent implements OnInit {

       // form
       search = false;
       stepKey = 'profile';

       selectedMaskId: number;
       selectedOriginId: number;

       output = '';
       
       recordTypes: number;
       
       refCodes: ReferenceCode[] = [];
       refCodesActive: ReferenceCode[] = [];
       refCode = new ReferenceCode();

       fileDelimeters: ReferenceCode[] = [];
       fileDelimetersActive: ReferenceCode[] = [];
       fileDelimeter = new ReferenceCode();

       fileCategories: ReferenceCode[] = [];
       fileCategoriesActive: ReferenceCode[] = [];
       fileCategory = new ReferenceCode();
       // ----------------------

       // validation  
       form: FormGroup;
       submitted: boolean;
       // ----------------------------------

       // grid details
       //  ----------------------------------
       applicationsOnEdit: Application[] = [];

       incomingFileMasks: IncomingFileMaskDescriptor[] = [];
       incomingFileMask: IncomingFileMask = new IncomingFileMask();

       fileProfiles: FileProfileDescriptor[] = [];
       fileProfile: FileProfile = new FileProfile();
       //  ----------------------------------

       // load handlers
       // -----------------------------------
       constructor(baseService: BaseService, private sessionService: FilemanSessionService, private fb: FormBuilder) {
              super(baseService);
        
              // Check permission on function calls in save method
              this.baseService.checkPermission('FileProfile');
              // -------------------------------------------------------------------
            
            // Override Create for query only
            if (this.baseService.getPage().toLowerCase() === 'query') {
                this.baseService.create = false;
            }
            // -------------------------------------------------------------------
       }

       ngOnInit() {
              // call to Permission Component --> componentInit();
              super.componentInit();

              // Retrieve values from session
              this.search = this.sessionService.checkSessionIdWithKey('mask');
              this.selectedOriginId = this.sessionService.getSessionIdWithKey('origin');
              this.selectedMaskId = this.sessionService.getSessionIdWithKey('mask');
              // -----------------------------------------------

              // form validation
              this.form = new FormGroup({
                     'id': new FormControl({disabled: true}),
                     'mask': new FormControl({}, Validators.required),
                     'category': new FormControl({},[Validators.required, Validators.maxLength(200)]),
                     'extension': new FormControl({},Validators.required),
                     'delimiter': new FormControl({},[Validators.required]),
                     'application': new FormControl({},Validators.required),
                     'compressed': new FormControl({},Validators.required),
                     'headings': new FormControl({},Validators.required),
                     'length': new FormControl({},Validators.required),
                     'multi': new FormControl({},Validators.required),
                     'status': new FormControl({},Validators.required)
              });

            // file type list
            this.getRefCodeDomainExtensionValues('FILE_EXTENSION');             
            this.getRefCodeDomainDelimiterValues('DELIMITER');  
            this.getRefCodeDomainCategoryValues('FILE_CATEGORY');       
            // -----------------------------------------------

              // Load data
              //if (!this.search) {
                     this.getFileMaskList(this.selectedOriginId);
              //}

              this.getGridList(this.selectedMaskId);              
              this.getApplicationsList();
              // -----------------------------------

       }
       // -----------------------------------

       // basic button events
       // -----------------------------------
       btn_back() {
            this.edit = false;
            this.form.reset();
            this.baseService.showAudit(false);
        }   

        btn_new() {
            this.edit = true;
            this.form.reset();
            this.baseService.showAudit(false);

              // Reset all items to null - empty inputs
              // ---------------------------------------
              this.fileProfile = new FileProfile();
              this.fileProfile.isActive = true;
              this.fileProfile.multipleRecordTypes = false;
              this.fileProfile.compressed = false;
              this.fileProfile.fixedLengthFile = false;
              this.fileProfile.containsColumnHeadings = false;
              this.fileProfile.consumingApplicationCMId = undefined;
              // ---------------------------------------
       }

       btn_save() {
              this.saveDetails();

              // If multiple record types are set, add another step in list
              this.sessionService.setSession(this.fileProfile.id, this.fileProfile.fileExtension.value, this.stepKey,
                     this.fileProfile.multipleRecordTypes);
              // ----------------------------------------------------------------------------------

              this.baseService.refreshPage();
       }

       btn_next_step() {

              // If multiple record types are set, add another step in list
              this.sessionService.setSession(this.fileProfile.id, this.fileProfile.fileExtension.value, this.stepKey,
                     this.fileProfile.multipleRecordTypes);
              // ----------------------------------------------------------------------------------

              this.baseService.refreshPage();
       }

       onSelectMask(incomingFileMaskId: number) {
              this.selectedMaskId = incomingFileMaskId;
              this.getGridList(this.selectedMaskId);
       }

       onSelectMaskEdit(incomingFileMaskId: number) {
              this.selectedMaskId = incomingFileMaskId;
              if (this.selectedMaskId) {
                     for (let i = 0; i < this.incomingFileMasks.length; i++) {
                            if (+this.selectedMaskId === this.incomingFileMasks[i].id) {
                                   //this.fileProfile.filenameMask = this.incomingFileMasks[i];
                                   break;
                            }
                     }
              }
       }
   
       btn_select_row(id: number) {

            // Determin object from grid row index
            this.fileProfile = this.fileProfiles[id] as FileProfile;
                
            if (this.baseService.getPage() === 'query') {

                    this.sessionService.setSession(this.fileProfile.id, this.fileProfile.fileExtension.value, this.stepKey);
                    this.baseService.refreshPage();

            } else {
                    this.getGridDetails(this.fileProfile.id);
            }

            // Audit information
            this.baseService.showAudit(true);
            this.baseService.auditTable = this.fileProfile.domainName
            this.baseService.auditId = this.fileProfile.id;
            // ------------------------------------------------------------------

        }

        btn_show_row(id: number) {
            this.fileProfile = this.fileProfiles[id] as FileProfile;

            this.output = JSON.stringify(this.fileProfile);
            this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
            this.output = this.output.replace(/":"/g, '</strong><i>');
            this.output = this.output.replace(/",/g, '</i><br>');
            this.output = this.output.replace(/,/g, '</i><br>');
            this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
            this.output = this.output.replace(/":/g, '</strong><i>');   
            this.output = this.output.replace(/"/g, "<strong>");
            this.output = this.output.replace(/{/g, '<br>');
            this.output = this.output.replace(/}/g, '</i>') + '</i><br>';

            this.detail = true;
        }

        btn_done() {
            this.detail = false;
        }
       // -----------------------------------

       // Data calls
       // -----------------------------------
       getRefCodeDomainExtensionValues(value: string) {
            this.baseService.getData('GET', 'referenceCodes/domain/' + value)
                .subscribe(
                    (data) => {
                        this.refCodes = data.json().content as ReferenceCode[];
                        
                        // create active list only
                        for (let i = 0; i < this.refCodes.length; i++) {
                            if (this.refCodes[i].isActive) {
                                this.refCodesActive.push(this.refCodes[i]);
                            }
                        }
                        // --------------------------------------------------------
                    }
                );
        }

        getRefCodeDomainDelimiterValues(value: string) {
            this.baseService.getData('GET', 'referenceCodes/domain/' + value)
                    .subscribe(
                        (data) => {
                            this.fileDelimeters = data.json().content as ReferenceCode[];
                            
                            // create active list only
                            for (let i = 0; i < this.fileDelimeters.length; i++) {
                                if (this.fileDelimeters[i].isActive) {
                                    this.fileDelimetersActive.push(this.fileDelimeters[i]);
                                }
                            }
                            // --------------------------------------------------------
                        }
                    );
            }

        getRefCodeDomainCategoryValues(value: string) {
            this.baseService.getData('GET', 'referenceCodes/domain/' + value)
                    .subscribe(
                        (data) => {
                            this.fileCategories = data.json().content as ReferenceCode[];
                            
                            // create active list only
                            for (let i = 0; i < this.fileCategories.length; i++) {
                                if (this.fileCategories[i].isActive) {
                                    this.fileCategoriesActive.push(this.fileCategories[i]);
                                }
                            }
                            // --------------------------------------------------------
                        }
                    );
            }

        getFileMaskList(fileOriginId?: number) {
              let endpoint = 'files/incomingFileMasks?size=600&sort=filenameMask';
              if (fileOriginId && fileOriginId != null) {
                     endpoint = 'files/incomingFileMasks/fileOrigin/' + fileOriginId + '?size=600&sort=priority';
              }
              
              this.baseService.getData('GET', endpoint)
                     .subscribe(
                            data => {
                                   this.incomingFileMasks = data.json().content as IncomingFileMaskDescriptor[];
                                   this.nodata = false;
                            }
                     );
       }

       getApplicationsList() {
              // const inputParams = [];
              // inputParams[0] = ['sort', 'name'];
              // const params = this.baseService.setParams(inputParams); // TODO : REFACTOR SORT SCENARIO!!

              this.baseService.getData('GET', 'applications/isActiveTrue?size=600&sort=name', null /* searchParams */, false /* paging */,
                     null /* updateParams */, true /* crossSectionOverride */, 'admin' /* crossSectionAPI */)
                     .subscribe(
                            (data) => {
                                   this.applicationsOnEdit = data.json().content as Application[];
                                   this.nodata = false;
                            }
                     );
       }

       getGridList(id?: number) {
              let endpoint = 'files/fileProfiles?size=600&sort=filenameMask,fileCategory.description';
              if (id && id != null) {
                     endpoint = 'files/fileProfiles/incomingFileMask/' + id + '?size=600&sort=filenameMask,fileCategory.description';
              }
              this.baseService.getData('GET', endpoint)
                     .subscribe(
                            data => {
                                   this.fileProfiles = data.json().content as FileProfileDescriptor[];
                                   this.nodata = false;
                                   this.edit = false;
                            }
                     );
       }

       getGridDetails(id?: number) {
           
              if (id && id != null) {
                     this.baseService.getData('GET', 'files/fileProfiles/' + id)
                            .subscribe(
                                   data => {
                                          this.fileProfile = data.json() as FileProfile;
                                          this.fileProfile.lastActionDate = new Date(this.fileProfile.lastActionDate);
                                          
                                          if (this.fileProfile.multipleRecordTypes) {
                                                 this.recordTypes = this.fileProfile.id;
                                          } else {
                                                 this.recordTypes = null;
                                          }

                                          try {
                                          this.form.get('id').setValue(this.fileProfile.id);
                                          this.form.get('mask').setValue(this.fileProfile.filenameMask);
                                          this.form.get('category').setValue(this.fileProfile.fileCategory.value);
                                          this.form.get('extension').setValue(this.fileProfile.fileExtension.value);                                          
                                          this.form.get('application').setValue(this.fileProfile.consumingApplicationCMId);
                                          this.form.get('compressed').setValue(this.fileProfile.compressed);
                                          this.form.get('headings').setValue(this.fileProfile.containsColumnHeadings);
                                          this.form.get('length').setValue(this.fileProfile.fixedLengthFile);
                                          this.form.get('multi').setValue(this.fileProfile.multipleRecordTypes);
                                          this.form.get('status').setValue(this.fileProfile.isActive);

                                        if (this.fileProfile.delimiter.value) {
                                            this.form.get( 'delimiter').setValue(this.fileProfile.delimiter.value);
                                        } else {
                                            this.form.get( 'delimiter').setValue('undefined');
                                        }

                                          } catch(e) {}
                                          this.nodata = false;
                                          this.edit = true;
                                   },
                                   (error: any) => {
                                       this.baseService.handleErrorLog('error', 'No detail records for this profile!', error);
                                   }
                            )
              }
       }

       saveDetails() {
            const method: string = this.baseService.checkMethod(this.fileProfile.id);
            const url: string = 'files/fileProfiles' + this.baseService.checkCreate(this.fileProfile.id);
              
            this.fileProfile = new FileProfile();

            // Get selected extension value
            for (let i = 0; i < this.refCodes.length; i++) {

                if (this.form.get('extension').value === this.refCodes[i].value) {
                    this.refCode = this.refCodes[i];
                }
            }
            // -------------------------------------------

            // Get selected file category
            for (let i = 0; i < this.fileCategories.length; i++) {

                if (this.form.get('category').value === this.fileCategories[i].value) {
                    this.fileCategory = this.fileCategories[i];
                }
            }
            // -------------------------------------------

            // Get selected delimiter
            for (let i = 0; i < this.fileDelimeters.length; i++) {

                if (this.form.get('delimiter').value === this.fileDelimeters[i].value) {
                    this.fileDelimeter = this.fileDelimeters[i];
                }
            }
            // -------------------------------------------

            // Get selected incoming file mask
            for (let i = 0; i < this.incomingFileMasks.length; i++) {

                if (this.form.get('mask').value === this.incomingFileMasks[i].filenameMask) {
                    this.fileProfile.incomingFileMask = this.incomingFileMasks[i];
                }
            }
            // -------------------------------------------

            // Get selected consuming application
            for (let i = 0; i < this.applicationsOnEdit.length; i++) {
                if (this.form.get('application').value === this.applicationsOnEdit[i].id) {
                    this.fileProfile.consumingApplicationCMId = this.applicationsOnEdit[i].id;
                }
            }
            // -------------------------------------------

            // set other form values
            this.fileProfile.id = this.form.get('id').value;
            this.fileProfile.fileCategory = this.fileCategory;            
            this.fileProfile.fileExtension = this.refCode;
            this.fileProfile.filenameMask = this.fileProfile.incomingFileMask.filenameMask;
            this.fileProfile.delimiter = this.fileDelimeter;
            this.fileProfile.compressed = this.form.get('compressed').value;
            this.fileProfile.containsColumnHeadings = this.form.get('headings').value;
            this.fileProfile.fixedLengthFile = this.form.get('length').value;
            this.fileProfile.multipleRecordTypes = this.form.get('multi').value;
            this.fileProfile.isActive = this.form.get('status').value;
            this.fileProfile.retentionDays = 0;
            // -------------------------------------------

            const jsonValue: string = JSON.stringify(this.fileProfile);
            const idMemory = (this.selectedMaskId) ? this.selectedMaskId : undefined;

            // console.error(JSON.stringify(this.fileProfile));

              // Update / Create items
              // -----------------------------------
              this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
                     .subscribe(
                            (data) => {
                                   this.baseService.handleErrorLog('success', 'Your File Profile was updated.', 'Saved');
                                   this.fileProfile = data.json() as FileProfile;
                                   this.getGridList(idMemory);
                                   this.edit = false;
                            },
                            (error: any) => {
                                   this.baseService.handleErrorLog('error', 'Saving of the File Profile failed!', error);
                            }
                     );
              // -----------------------------------

       }
       // -----------------------------------
}

