import { Component, Input, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
import { SelectItem } from 'primeng/api';
// ------------------

// Fileman Common
import { FilemanSession } from '@fileman/index-domain';
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { IncomingFileMask, IncomingFileMaskDescriptor, FileOriginDescriptor, ReferenceCode } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './incoming-file-masks.component.html'
})

export class MasksComponent extends PermissionComponent implements OnInit {

    // form
    fileOrigins: FileOriginDescriptor[] = [];
    fileOrigin = new FileOriginDescriptor();
    fileOriginId: number = undefined;

    output = '';

    search = false;
    stepKey = 'mask';

    refCodes: ReferenceCode[] = [];
    refCodesActive: ReferenceCode[] = [];
    refCode = new ReferenceCode();
    // ----------------------

    // validation  
    form: FormGroup;
    submitted: boolean;
    // ----------------------------------
        
    // grid details
    // ----------------------------------
    incomingFileMasks: IncomingFileMaskDescriptor[] = [];
    incomingFileMask: IncomingFileMask = new IncomingFileMask();
    fileExt = new ReferenceCode();
    //  ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private sessionService: FilemanSessionService, private fb: FormBuilder) {
        super(baseService);
        
        // Check permission on function calls in save method
        this.baseService.checkPermission('IncomingFileMask');
        // -------------------------------------------------------------------
        
        // Override Create for query only
        if (this.baseService.getPage().toLowerCase() === 'query') {
            this.baseService.create = false;
        }
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // Retrieve values from session
        this.search = this.sessionService.checkSessionIdWithKey('origin');
        this.fileOriginId = this.sessionService.getSessionIdWithKey('origin');
        // -----------------------------------------------

        // file type list
        this.getRefCodeDomainSelectValues();        
        // -----------------------------------------------

        // form validation
        this.form = new FormGroup({
            'id': new FormControl({disabled: true}),
            'origin': new FormControl({}, [Validators.required, Validators.maxLength(50)]),
            'mask': new FormControl({}, [Validators.required, Validators.maxLength(400)]),
            'extension': new FormControl({}, [Validators.required, Validators.maxLength(50)]),
            'compressed': new FormControl({}, [Validators.required]),
            'priority': new FormControl({}, [Validators.required]),
            'status': new FormControl({}, [Validators.required])
        });

        // Load data
        this.getFileOriginsList();
        this.getGridList(this.fileOriginId);
        // -----------------------------------

    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_back() {
        this.edit = false;
        this.form.reset();
        this.baseService.showAudit(false);
    }

    btn_new() {
        this.edit = true;
        this.form.reset();
        this.baseService.showAudit(false);

        // Reset all items to null - empty inputs
        // ---------------------------------------
        this.incomingFileMask = new IncomingFileMask();
        this.incomingFileMask.isActive = true;
        if (this.fileOriginId) {
            for (let i = 0; i < this.fileOrigins.length; i++) {
                if (+this.fileOriginId === this.fileOrigins[i].id) {
                    this.incomingFileMask.fileOrigin = new FileOriginDescriptor();
                    this.incomingFileMask.fileOrigin.id = this.fileOrigins[i].id;
                    this.incomingFileMask.fileOrigin.isActive = this.fileOrigins[i].isActive;
                    this.incomingFileMask.fileOrigin.origin = this.fileOrigins[i].origin;
                    this.incomingFileMask.fileOrigin.module = this.fileOrigins[i].module;
                    this.incomingFileMask.fileOrigin.autoLoad = this.fileOrigins[i].autoLoad;
                    this.incomingFileMask.fileOrigin.purpose = this.fileOrigins[i].purpose;
                    break;
                }
            }
        }
        // ---------------------------------------
    }

    btn_save() {
      this.saveDetails();

      this.sessionService.setSession(this.incomingFileMask.id, this.incomingFileMask.filenameMask, this.stepKey);
      this.baseService.refreshPage();

    }

    btn_next_step() {
        
      this.sessionService.setSession(this.incomingFileMask.id, this.incomingFileMask.filenameMask, this.stepKey);
      this.baseService.refreshPage();

    }
      
    determineModelFileOriginId(): number {
        return (this.incomingFileMask.fileOrigin) ? this.incomingFileMask.fileOrigin.id : undefined;
    }

    onSelectFileOrigin(fileOriginId: number) {
        this.fileOriginId = fileOriginId;
        this.getGridList(this.fileOriginId);
    }
   
    btn_select_row(id: number) {

        // Determin object from grid row index
        this.incomingFileMask = this.incomingFileMasks[id] as IncomingFileMask;
        
        if (this.baseService.getPage() === 'query') {

            this.sessionService.setSession(this.incomingFileMask.id, this.incomingFileMask.filenameMask, this.stepKey);
            this.baseService.refreshPage();

        } else {
            this.getGridDetails(this.incomingFileMask.id);
        }

        // Audit information
        this.baseService.showAudit(true);
        this.baseService.auditTable = this.incomingFileMask.domainName
        this.baseService.auditId = this.incomingFileMask.id;
        // ------------------------------------------------------------------

   }

   btn_show_row(id: number) {
        this.incomingFileMask = this.incomingFileMasks[id] as IncomingFileMask;

        this.output = JSON.stringify(this.incomingFileMask);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
        this.output = this.output.replace(/":/g, '</strong><i>');   
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';

        this.detail = true;
   }

   btn_done() {
        this.detail = false;
   }

   onSliderChange(selected) {
       this.incomingFileMask.priority = +selected.value;
   }
    // -----------------------------------

    // Data calls
    // -----------------------------------    
    getRefCodeDomainSelectValues() {
        this.baseService.getData('GET', 'referenceCodes/domain/FILE_EXTENSION')
                .subscribe(
                    (data) => {
                        this.refCodes = data.json().content as ReferenceCode[];

                        // create active list only
                        for (let i = 0; i < this.refCodes.length; i++) {
                            if (this.refCodes[i].isActive) {
                                this.refCodesActive.push(this.refCodes[i]);
                            }
                        }
                        // --------------------------------------------------------

                        this.nodata = false;
                    }
                );
    }

    getFileOriginsList() {
        this.baseService.getData('GET', 'files/fileOrigins?size=600&sort=origin')
            .subscribe(
                (data) => {
                    this.fileOrigins = data.json().content as FileOriginDescriptor[];
                    this.nodata = false;
                }
            );
    }

    getGridList(id?: number) {
        let endpoint = 'files/incomingFileMasks?size=600&sort=priority,filenameMask';
        if (id && id != null) {
            endpoint = 'files/incomingFileMasks/fileOrigin/' + id + '?size=600&sort=priority,filenameMask';
        }

        this.baseService.getData('GET', endpoint)
            .subscribe(
                data => {
                    this.incomingFileMasks = data.json().content as IncomingFileMaskDescriptor[];
                    this.nodata = false;
                    this.edit = false;
                }
            );
    }

    getGridDetails(id: number) {
        if (id && id != null) {
            this.baseService.getData('GET', 'files/incomingFileMasks/' + id)
                .subscribe(
                    data => {
                        this.incomingFileMask = data.json() as IncomingFileMask;

                        this.form.get( 'id').setValue(this.incomingFileMask.id);
                        this.form.get( 'origin').setValue(this.incomingFileMask.fileOrigin.id);
                        this.form.get( 'mask').setValue(this.incomingFileMask.filenameMask);
                        this.form.get( 'extension').setValue(this.incomingFileMask.fileExtension.value);
                        this.form.get( 'compressed').setValue(this.incomingFileMask.compressed);
                        this.form.get( 'priority').setValue(this.incomingFileMask.priority);
                        this.form.get( 'status').setValue(this.incomingFileMask.isActive);

                        this.nodata = false;
                        this.edit = true;
                    }
                );
        }
    }

    saveDetails() {
        const method: string = this.baseService.checkMethod(this.incomingFileMask.id);
        const url: string = 'files/incomingFileMasks' + this.baseService.checkCreate(this.incomingFileMask.id);

        this.incomingFileMask = new IncomingFileMask();
        
        for (let i = 0; i < this.refCodes.length; i++) {

            if (this.form.get('extension').value === this.refCodes[i].value) {
                this.refCode = this.refCodes[i];
            }
        }

        for (let i = 0; i < this.fileOrigins.length; i++) {

            if (this.form.get('origin').value === this.fileOrigins[i].id) {
                this.fileOrigin = this.fileOrigins[i];
            }
        }

        this.incomingFileMask.id = this.form.get('id').value;
        this.incomingFileMask.fileOrigin = this.fileOrigin;
        this.incomingFileMask.filenameMask = this.form.get('mask').value;
        this.incomingFileMask.fileExtension = this.refCode;
        this.incomingFileMask.compressed = this.form.get('compressed').value;
        this.incomingFileMask.isActive = this.form.get('status').value;
        this.incomingFileMask.priority =  this.form.get('priority').value;

        const jsonValue: string = JSON.stringify(this.incomingFileMask);
        const idMemory = (this.fileOriginId) ? this.fileOriginId : undefined;

        // Update / Create items
        this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
            .subscribe(
                (data) => {
                    this.baseService.handleErrorLog('success', 'Your Incoming File Mask was updated.', 'Saved');
                    this.incomingFileMask = data.json() as IncomingFileMask;
                    this.getGridList(idMemory);
                    this.edit = false;
                },
                (error: any) => {
                    this.baseService.handleErrorLog('error', 'Saving of the Incoming File Mask failed!', error);
                }
            );
        // -----------------------------------
    }
    // -----------------------------------
}

