import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';
import { SelectItem } from 'primeng/api';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Load other
//  -----------------
import { ReferenceCode } from '@fileman/index-domain';

@Component({
    templateUrl: './reference-codes.component.html'
})

export class ReferenceCodesComponent extends PermissionComponent implements OnInit {

    // select domain dropdown
    // ----------------------------------
    refCodeDomains: String[] = [];
    refCodeDomainList: SelectItem[];    
    domainSelection = '';
    // ----------------------------------

    // validation  
    form: FormGroup;
    submitted: boolean;
    output = '';

    newDomain = false;
    originalText = '';
    // ----------------------------------
    
    // grid details
    //  ----------------------------------
    refCodes: ReferenceCode[] = [];
    refCode: ReferenceCode = new ReferenceCode();
    //  ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private fb: FormBuilder) {
        super(baseService);

        // Check permission on function calls in save method
        this.baseService.checkPermission('ReferenceCode');
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // call to local getRefCodeDomainSelectValues();
        this.getRefCodeDomainSelectValues();
        
        // form validation
        this.form = new FormGroup({
            'id': new FormControl({disabled: true}),
            'domain': new FormControl([Validators.required, Validators.maxLength(100)]),
            'value': new FormControl([Validators.required, Validators.maxLength(300)]),
            'description': new FormControl([Validators.required, Validators.maxLength(500)]),
            'isActive': new FormControl([Validators.required])
        });

        this.domainSelection = '';
        this.getGridList(null);
    }

    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_done() {
        this.detail = false;
    }

    btn_back() {
        this.edit = false;
        this.form.reset();
        this.baseService.showAudit(false);
        this.newDomain = false;
        
        this.onSelectDomain(this.domainSelection);
    }

    btn_new() {
        this.edit = true;              
        this.form.reset();
        this.baseService.showAudit(false);
        this.newDomain = false;

        this.refCode = new ReferenceCode();
        if (this.domainSelection && this.domainSelection != null) {
            this.refCode.domain = this.domainSelection;
            this.form.get('domain').setValue(this.refCode.domain);
        }
        this.refCode.isActive = true;
        // ---------------------------------------
    }

    btn_save() {
      this.saveDetails();
    }
    // -----------------------------------

    // basic select events
    // -----------------------------------
    btn_select_row(index: number) {
        
        // Audit information
        this.baseService.showAudit(true);
        this.baseService.auditTable = this.refCodes[index].domainName
        this.baseService.auditId = this.refCodes[index].id;
        // ------------------------------------------------------------------

        // Determine object from grid row index
        this.getGridDetails(this.refCodes[index].id);
    }

    btn_show_row(index: number) {
        this.refCode = this.refCodes[index] as ReferenceCode;
        this.output = JSON.stringify(this.refCode);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");
        this.output = this.output.replace(/":/g, '</strong><i>');
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
    }

    determineModelDomain(): string {
        return this.form.get('domain').value;
    }

    determineDomainSelection(): string {
         return this.domainSelection;
    }

    onSelectDomain(domainValue: string) {
        if (domainValue && domainValue !== '-- Filter by Domain --' && domainValue !== '') {
            this.domainSelection = domainValue;
            this.getGridList(domainValue);
        } else {
            this.domainSelection = '';
            this.getGridList(null);
        } 
    }

    onSelectDomainAdd(domain: string) {       
                
        if (domain === this.originalText) { // No new typed values in this box
            if (this.refCode && this.refCode != null) {
                this.refCode.domain =  domain;
            }
            this.newDomain = false;
        } else { //User typed in this box and a new domain needs to be created
            this.newDomain = true;
        }

    }

    // -----------------------------------

    // Data calls
    // -----------------------------------
    getRefCodeDomainSelectValues() {
        this.baseService.getData('GET', 'referenceCodes/domain')
                .subscribe(
                    (data) => {
                        this.refCodeDomains = data.json() as String[];
                        this.refCodeDomainList = [];
                        for (let i=0; i<this.refCodeDomains.length; i++) {
                            this.refCodeDomainList.push({label: this.refCodeDomains[i].toString(), value: this.refCodeDomains[i].toString()})
                        }                        
                        this.nodata = false;
                    }
                );
    }

    getGridList(domain: string) {
        let endpoint = 'referenceCodes?size=600&sort=domain,value';
        let domainPresent = false;
        const domainMemory = domain;
        if (domain && domain != null && domain !== '') {
            domainPresent = true;
            endpoint = 'referenceCodes/domain/' + domain + '?size=600&sort=domain,value';
        }        
        this.baseService.getData('GET', endpoint)
            .subscribe(
                (data) => {
                    this.refCodes = data.json().content as ReferenceCode[];
                    if (domainPresent) {
                        this.domainSelection = domainMemory;
                    }
                    this.nodata = false;
                }
            );
    }

    getGridDetails(id?: number) {
        if (id && id != null) {
            this.baseService.getData('GET', 'referenceCodes/' + id)
                .subscribe(
                    (data) => {
                        this.refCode = data.json() as ReferenceCode;

                        this.newDomain = false;

                        this.form.get('id').setValue(this.refCode.id);
                        this.form.get('domain').setValue(this.refCode.domain);
                        this.form.get('value').setValue(this.refCode.value);
                        this.form.get('description').setValue(this.refCode.description);
                        this.form.get('isActive').setValue(this.refCode.isActive);
                        
                        if (this.refCode.domain) {
                            this.form.get( 'domain').setValue(this.refCode.domain);
                        } else {
                            this.form.get( 'domain').setValue('undefined');
                        }

                        this.originalText = this.refCode.domain;

                        this.nodata = false;
                        this.edit = true;
                    }
                );
        }
    }

    saveDetails() {

        this.refCode.id = this.form.get('id').value;
        
        let parDomain = '';
        parDomain = this.form.get('domain').value;
        parDomain = parDomain.toUpperCase();

        this.refCode.domain = parDomain;
        this.refCode.value = this.form.get('value').value;
        this.refCode.description = this.form.get('description').value;
        
        // save new, remove id so that it creates a new one
        if (this.newDomain) {
            this.refCode.id = null;
        }
       // -----------------------------------

        const method: string = this.baseService.checkMethod(this.refCode.id);
        const url: string = 'referenceCodes' + this.baseService.checkCreate(this.refCode.id);
        const jsonValue: string = JSON.stringify(this.refCode);

        // Update / Create items
        // -----------------------------------
        this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
           .subscribe(
               (data) => {
                    this.baseService.handleErrorLog('success', 'Your Reference Code was saved.', 'Saved');
                    this.refCode = data.json() as ReferenceCode;
                    this.getGridList(this.refCode.domain);
                    this.getRefCodeDomainSelectValues();
                    this.edit = false;
               },
               (error: any) => {
                    this.baseService.handleErrorLog('error', 'Saving of the Reference Code failed!', error);
               }
           );
        // -----------------------------------
    }
    // -----------------------------------
}
