import { Component, OnInit } from '@angular/core';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Load other
import { TableColumn, TableColumnDescriptor, TableDescriptor, Table } from '@fileman/index-domain';
//  -----------------

@Component({
  templateUrl: './table-columns.component.html'
})

export class TableColumnsComponent extends PermissionComponent implements OnInit {

    // grid details
    //  ----------------------------------
    tables: TableDescriptor[] = [];
    table: Table = new Table();

    tableColumns: TableColumnDescriptor[] = [];
    tableColumn: TableColumn = new TableColumn();
    //  ----------------------------------

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService) {
        super(baseService);
    }

    ngOnInit() {
        // Call to Permission Component --> componentInit();
        super.componentInit();

        // Call to getTablesList();
        this.getTablesList();

        // Call to local getGridList();
        this.getGridList();
    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_back() {
        this.edit = false;
    }

    btn_new() {
        this.edit = true;

        // Reset all items to null - empty inputs
        // ---------------------------------------
        this.tableColumn = new TableColumn();
        // ---------------------------------------
    }

    btn_save() {
      this.saveDetails();
    }

    onRowSelect() {
       this.getGridDetails(this.tableColumn.id);
    }

    onSelectTable(id: number) {
        this.getGridList(id);
    }
    // -----------------------------------

    // Data calls
    // -----------------------------------
    getTablesList() {
        this.baseService.getData('GET', 'tables?size=600&sort=owner,tableName')
            .subscribe(
                (data) => {
                    this.tables = data.json().content as TableDescriptor[];
                    this.nodata = false;
                }
            );
    }

    getGridList(tableId?: number) {
        if (tableId && tableId != null) {
            this.baseService.getData('GET', 'tables/' + tableId)
                .subscribe(
                    (data) => {
                        this.table = data.json() as Table;
                        this.tableColumns = this.table.tableColumns as TableColumnDescriptor[];
                        this.nodata = false;
                    }
                );
        }
    }

    determineModelTableId(): number {
        return (this.tableColumn.table) ? this.tableColumn.table.id : undefined;
    }

    getGridDetails(tableColId?: number) {
        if (tableColId && tableColId != null) {
            this.baseService.getData('GET', 'tableColumns/' + tableColId)
                .subscribe(
                    (data) => {
                        this.tableColumn = data.json() as TableColumn;
                        this.nodata = false;
                        this.edit = true;
                    }
                );
        }
    }

    saveDetails() {
        const method: string = this.baseService.checkMethod(this.tableColumn.id);
        const url: string = 'tableColumns' + this.baseService.checkCreate(this.tableColumn.id);
        const jsonValue: string = JSON.stringify(this.tableColumn);
        const idMemory = (this.table.id) ? this.table.id : undefined;

        // Update / Create items
        this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
            .subscribe(
                (data) => {
                    this.baseService.handleErrorLog('success', 'Your Table Column was saved.', 'Saved');
                    this.tableColumn = data.json() as TableColumn;
                    this.getGridList(idMemory);
                    this.edit = false;
                },
                (error: any) => {
                    this.baseService.handleErrorLog('error', 'Saving of the Table Column failed!', error);
                }
            );
        // -----------------------------------
    }
    // -----------------------------------
}
