import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { TableProfile, TableProfileDescriptor, FileProfileDescriptor, FileProfile } from '@fileman/index-domain';
//  -----------------

@Component({
       templateUrl: './table-profiles.component.html'
})

export class TableProfilesComponent extends PermissionComponent implements OnInit {

       // form
       search = false;
       stepKey = 'audit';

       profileId: number = undefined;
       // ----------------------

       // validation  
       form: FormGroup;
       submitted: boolean;
       // ----------------------------------

       // grid details
       //  ----------------------------------
       fileProfiles: FileProfileDescriptor[] = [];
       fileProfile: FileProfile = new FileProfile();

       tableProfiles: TableProfileDescriptor[] = [];
       tableProfile: TableProfile = new TableProfile();
       //  ----------------------------------

       // load handlers
       // -----------------------------------
       constructor(baseService: BaseService, private sessionService: FilemanSessionService, private fb: FormBuilder) {
              super(baseService);
       }

       ngOnInit() {
              // Call to Permission Component --> componentInit();
              super.componentInit();

              // Retrieve values from session
              this.search = this.sessionService.checkSessionIdWithKey(this.stepKey);
              this.profileId = this.sessionService.getSessionIdWithKey(this.stepKey);
              // -----------------------------------------------

              // form validation
              this.form = new FormGroup({
                     'id': new FormControl({disabled: true}),
                     'threshold': new FormControl({}, Validators.required),
                     'profile': new FormControl({}, Validators.required)
              });

              // Load data
              this.getFileProfilesList();
              this.getGridList(this.profileId);
              // -----------------------------------------------

       }
       // -----------------------------------

       // basic button events
       // -----------------------------------
       btn_back() {
              this.edit = false;
       }

       btn_new() {
              this.edit = true;

              // Reset all items to null - empty inputs
              // ---------------------------------------
              this.form.get('id').setValue('');
              this.form.get('threshold').setValue('');
              this.form.get('profile').setValue('');

              this.tableProfile = new TableProfile();
              // ---------------------------------------
       }

       btn_save() {
              this.saveDetails();
       }

       onRowSelect() {
              this.getGridDetails(this.tableProfile.id);
       }

       onSelectFileProfile(id: number) {
              this.getGridList(id);
       }
       // -----------------------------------

       // Data calls
       // -----------------------------------
       getFileProfilesList() {
              this.baseService.getData('GET', 'files/fileProfiles?size=600&sort=filenameMask,fileCategory.description')
                     .subscribe(
                            (data) => {
                                   this.fileProfiles = data.json().content as FileProfileDescriptor[];
                                   this.nodata = false;
                            }
                     );
       }

       getGridList(fileProfileId?: number) {
              if (fileProfileId && fileProfileId != null) {
                     this.baseService.getData('GET', 'files/fileProfiles/' + fileProfileId)
                            .subscribe(
                                   (data) => {
                                          this.fileProfile = data.json() as FileProfile;
                                          this.tableProfiles = this.fileProfile.tableProfiles as TableProfileDescriptor[];
                                          this.nodata = false;
                                   }
                            );
              }
       }

       determineModelFileProfileId(): number {
              return (this.tableProfile.fileProfile) ? this.tableProfile.fileProfile.id : undefined;
       }

       getGridDetails(tableProfileId?: number) {
              if (tableProfileId && tableProfileId != null) {
                     this.baseService.getData('GET', 'tableProfiles/' + tableProfileId)
                            .subscribe(
                                   (data) => {
                                          this.tableProfile = data.json() as TableProfile;

                                          this.form.get('id').setValue(this.tableProfile.id);
                                          this.form.get('threshold').setValue(this.tableProfile.thresholdPercentage);
                                          this.form.get('profile').setValue(this.tableProfile.fileProfile);

                                          this.nodata = false;
                                          this.edit = true;
                                   }
                            );
              }
       }

       saveDetails() {
              const method: string = this.baseService.checkMethod(this.tableProfile.id);
              const url: string = 'tableProfiles' + this.baseService.checkCreate(this.tableProfile.id);
              const jsonValue: string = JSON.stringify(this.tableProfile);
              const idMemory = (this.fileProfile.id) ? this.fileProfile.id : undefined;

              // Update / Create items
              this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
                     .subscribe(
                            (data) => {
                                   this.baseService.handleErrorLog('success', 'Your Table Profile was saved.', 'Saved');
                                   this.tableProfile = data.json() as TableProfile;
                                   this.getGridList(idMemory);
                                   this.edit = false;
                            },
                            (error: any) => {
                                   this.baseService.handleErrorLog('error', 'Saving of the Table Profile failed!', error);
                            }
                     );
              // -----------------------------------
       }
       // -----------------------------------
}
