import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { Table, TableDescriptor, TableColumnDescriptor } from '@fileman/index-domain';
//  -----------------

@Component({
       templateUrl: './tables.component.html'
})
export class TablesComponent extends PermissionComponent implements OnInit {


       // form
       //  ----------------------------------
       search = false;
       stepKey = 'table';
       output = '';
       //  ----------------------------------

       // validation  
       form: FormGroup;
       submitted: boolean;
       // ----------------------------------

       // grid details
       //  ----------------------------------
       tables: TableDescriptor[] = [];
       table: Table = new Table();
       //  ----------------------------------

       isTableColumns = false;

       // load handlers
       // -----------------------------------
       constructor(baseService: BaseService, private sessionService: FilemanSessionService, private fb: FormBuilder) {
              super(baseService);

              // Check permission on function calls in save method
              this.baseService.checkPermission('table');
              // -------------------------------------------------------------------
       }

       ngOnInit() {
              // call to Permission Component --> componentInit();
              super.componentInit();

              // Retrieve values from session
              this.search = this.sessionService.checkSessionIdWithKey(this.stepKey);
              // -----------------------------------------------

              // form validation
              this.form = new FormGroup({
                     'id': new FormControl({disabled: true}),
                     'table': new FormControl(Validators.compose([Validators.required, Validators.maxLength(30)]))
              });

              // Load data
              this.getGridList();
              // -----------------------------------

       }
       // -----------------------------------

       // basic button events
       // -----------------------------------
       btn_back() {
            this.edit = false;
            this.form.reset();
            this.baseService.showAudit(false);
        }

        btn_new() {
            this.edit = true;
            this.form.reset();
            this.baseService.showAudit(false);
            this.isTableColumns = false;

            // Reset all items to null - empty inputs
            // ---------------------------------------
            this.table = new Table();
            // ---------------------------------------
       }

        btn_save() {
              this.saveDetails();

              this.sessionService.setSession(this.table.id, this.table.tableName, this.stepKey);
              this.baseService.refreshPage();

        }
   
        btn_select_row(id: number) {

            // Determin object from grid row index
            this.table = this.tables[id] as Table;
            
            // Audit information
            this.baseService.showAudit(true);
            this.baseService.auditTable = this.table.domainName
            this.baseService.auditId = this.table.id;
            // ------------------------------------------------------------------

            if (this.baseService.getPage() === 'query') {

                this.sessionService.setSession(this.table.id, this.table.tableName, this.stepKey);
                this.baseService.refreshPage();

            } else {
                    this.getGridDetails(this.table.id);
            }

    }

    btn_show_row(id: number) {
        this.table = this.tables[id] as Table;

        this.output = JSON.stringify(this.table);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');            
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");        
        this.output = this.output.replace(/":/g, '</strong><i>');   
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';

        this.detail = true;
    }

    btn_done() {
        this.detail = false;
    }
       
    // -----------------------------------

       // Data calls
       // -----------------------------------
       getGridList() {
              this.baseService.getData('GET', 'tables?size=600&sort=owner,tableName')
                     .subscribe(
                            (data) => {
                                   this.tables = data.json().content as TableDescriptor[];
                                   this.nodata = false;
                            }
                     );
       }

       determineIsTableColumns(tableColumns: TableColumnDescriptor[]): boolean {
              return (tableColumns && tableColumns.length > 0);
       }

       getGridDetails(tableId?: number) {
              if (tableId && tableId != null) {
                     this.baseService.getData('GET', '/tables/' + tableId)
                            .subscribe(
                                   (data) => {
                                          this.table = data.json() as Table;

                                          this.form.get('id').setValue(this.table.id);
                                          this.form.get('table').setValue(this.table.tableName);

                                          this.nodata = false;
                                          this.edit = true;
                                          this.isTableColumns = this.determineIsTableColumns(this.table.tableColumns);
                                   }
                            );
              }
       }

       saveDetails() {
              const method: string = this.baseService.checkMethod(this.table.id);
              const url: string = 'tables' + this.baseService.checkCreate(this.table.id);

              this.table = new Table();

              this.table.id = this.form.get('id').value;
              this.table.tableName = this.form.get('table').value;

              const jsonValue: string = JSON.stringify(this.table);

              // Update / Create items
              this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
                     .subscribe(
                            (data) => {
                                   this.baseService.handleErrorLog('success', 'Your Table was saved.', 'Saved');
                                   this.table = data.json() as Table;
                                   this.getGridList();
                                   this.edit = false;
                                   this.isTableColumns = this.determineIsTableColumns(this.table.tableColumns);
                            },
                            (error: any) => {
                                   this.baseService.handleErrorLog('error', 'Saving of the Table failed!', error);
                            }
                     );
              // -----------------------------------
       }
       // -----------------------------------
}
