import { Component, OnInit, ViewChild, ViewContainerRef, ComponentFactoryResolver } from '@angular/core';
import { MenuItem } from 'primeng/primeng';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fielman Common
import { FilemanSessionService } from '@fileman/index-service';
import { FilemanSession } from '@fileman/index-domain';
// ------------------

// Load other
import { OriginsComponent } from '@fileman/file/components/file-origins.component';
import { MasksComponent } from '@fileman/file/components/incoming-file-masks.component';
import { TablesComponent } from '@fileman/table/components/tables.component';
import { ProfilesComponent } from '@fileman/file/components/file-profiles.component';
import { TypesComponent } from '@fileman/wizzard/components/types.component';
//  -----------------

@Component({
  templateUrl: './configuration.component.html'
})

export class ConfigurationComponent extends PermissionComponent implements OnInit {

    // form
    session: FilemanSession[] = [];
    // ----------------------

    // step details
    //  ----------------------------------
    activeIndex = 0;
    //  ----------------------------------

    // grid details
    //  ----------------------------------

    //  ----------------------------------

    // Include controls
    @ViewChild('subcomp', { read: ViewContainerRef }) tab: ViewContainerRef;

    // load handlers
    // -----------------------------------
    constructor(private resolver: ComponentFactoryResolver, baseService: BaseService, private filemanService: FilemanSessionService) {
        super(baseService);

    }

    ngOnInit() {

        // call to Permission Component --> componentInit();
        super.componentInit();

        // Get current session values
        this.session = this.filemanService.getSessionFull();
        // -----------------------------------

        // Determin active index
        this.activeIndex = this.filemanService.getSessionIdCurrentActive();
        // -----------------------------------

        // What components should be loaded for each step
        if (this.activeIndex === 0) {
            this.tab.clear();
            this.tab.createComponent(this.resolver.resolveComponentFactory(OriginsComponent)).changeDetectorRef.detectChanges();
        } else if (this.activeIndex === 1) {
            this.tab.clear();
            this.tab.createComponent(this.resolver.resolveComponentFactory(MasksComponent)).changeDetectorRef.detectChanges();
        } else if (this.activeIndex === 2) {
            this.tab.clear();
            this.tab.createComponent(this.resolver.resolveComponentFactory(ProfilesComponent)).changeDetectorRef.detectChanges();
        } else if (this.activeIndex === 3) {
            this.tab.clear();
            this.tab.createComponent(this.resolver.resolveComponentFactory(TypesComponent)).changeDetectorRef.detectChanges();
        }
        // -----------------------------------------------

    }
    // -----------------------------------
}

