import { Component, OnInit, ViewChild, ViewContainerRef, ComponentFactoryResolver } from '@angular/core';
import { MenuItem } from 'primeng/primeng';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fielman Common
import { FilemanSessionService } from '@fileman/index-service';
import { FilemanSession } from '@fileman/index-domain';
// ------------------

// Load other
import { TablesComponent } from '@fileman/table/components/tables.component';
import { RecordTypesComponent } from '@fileman/record/components/record-types.component';
//  -----------------

@Component({
  templateUrl: './types.component.html'
})

export class TypesComponent extends PermissionComponent implements OnInit {

    showRecordTypes = false;

    // Include controls
    @ViewChild('subCompTypeTable', { read: ViewContainerRef }) table: ViewContainerRef;
    @ViewChild('subCompTypeRecords', { read: ViewContainerRef }) record: ViewContainerRef;

    // load handlers
    // -----------------------------------
    constructor(private resolver: ComponentFactoryResolver, baseService: BaseService, private filemanService: FilemanSessionService) {
        super(baseService);

    }

    ngOnInit() {

        // call to Permission Component --> componentInit();
        super.componentInit();

        // What components should be loaded for each step        
        this.table.clear();
        this.table.createComponent(this.resolver.resolveComponentFactory(TablesComponent)).changeDetectorRef.detectChanges();
        
        if (this.filemanService.checkSessionAdditionalStepWithKey('record')) {
            //this.showRecordTypes = true;
            this.record.clear();
            this.record.createComponent(this.resolver.resolveComponentFactory(RecordTypesComponent)).changeDetectorRef.detectChanges();
        }        
        // -----------------------------------------------

    }
    // -----------------------------------

}

